local addonName, addonTab = ...

-- ##################### --
-- SavedVariables Loader --
-- ##################### --

addonTab.defaults = {}

addonTab.loader = CreateFrame("Frame")
	addonTab.loader:RegisterEvent("ADDON_LOADED")
	addonTab.loader:SetScript("OnEvent", function(self, event, ...)
		addonTab.loader[event](self, ...) -- call event handlers
	end)

function addonTab.loader:ADDON_LOADED(addonName)
	if not (_G[addonName.."DBPC"]) then -- first time the addon is used
		print(addonName..": First time use, creating saved variables")
		_G[addonName.."DBPC"] = {} -- allocate a table to store your saved variables (the table will be saved to disk on logout)
		-- Now, load the table with defaults if needed.
		_G[addonName.."DBPC"] = addonTab.defaults
	end
	-- Initialise the addon with the Saved Variables information (new or loaded from disk)
	addonTab.db = _G[addonName.."DBPC"]
	self:UnregisterEvent("ADDON_LOADED")
end

addonTab.panel = CreateFrame( "Frame", addonName.."Panel", InterfaceOptionsFramePanelContainer );
-- Register in the Interface Addon Options GUI
-- Set the name for the Category for the Options Panel
addonTab.panel.name = addonName;
-- Add the panel to the Interface Options
InterfaceOptions_AddCategory(addonTab.panel);

-- Panel Title
local dvtitle=CreateFrame("Frame", "dvtitle", DejaViewClassicPanel)
	dvtitle:SetPoint("TOPLEFT", 5, -5)
	dvtitle:SetScale(2.0)
	dvtitle:SetWidth(150)
	dvtitle:SetHeight(50)
	dvtitle:Show()

local dvtitleFS = dvtitle:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	dvtitleFS:SetText('|cff00c0ffDejaViewClassic|r')
	dvtitleFS:SetPoint("TOPLEFT", 0, 0)
	dvtitleFS:SetFont("Fonts\\FRIZQT__.TTF", 10)
	
dvresetcheck = CreateFrame("Button", "DVResetButton", DejaViewClassicPanel, "UIPanelButtonTemplate")
	dvresetcheck:ClearAllPoints()
	dvresetcheck:SetPoint("BOTTOMLEFT", 5, 5)
	dvresetcheck:SetScale(1.25)
	dvresetcheck:SetWidth(125)
	dvresetcheck:SetHeight(30)
	dvresetcheck.tooltipText = 'Reset DejaViewClassic options to default settings.' --Creates a tooltip on mouseover.
	_G[dvresetcheck:GetName() .. "Text"]:SetText("Reset to Default")
	
	dvresetcheck:SetScript("OnClick", function (self, button, down)
 		_G[addonName.."DBPC"] = addonTab.defaults
		addonTab.db = _G[addonName.."DBPC"]
		ReloadUI();
end)

 dvlockcheck = CreateFrame("CheckButton", addonName.."LockAll", DejaViewClassicPanel, "InterfaceOptionsCheckButtonTemplate")
	dvlockcheck:ClearAllPoints()
	dvlockcheck:SetPoint("TOPLEFT", 25, -35)
	dvlockcheck:SetScale(1.25)
	_G[dvlockcheck:GetName() .. "Text"]:SetText("Lock all frames")
	dvlockcheck.tooltipText = 'Checked locks all DejaViewClassic frames. Unchecked unlocks all DejaViewClassic frames.' --Creates a tooltip on mouseover.
	dvlockcheck:SetChecked(true)
	
	dvlockcheckframe=CreateFrame("Frame", "dvlockcheckframe", UIParent)
	dvlockcheckframe:SetClampedToScreen(1)
	dvlockcheckframe:SetFrameStrata(BACKGROUND)
	dvlockcheckframe:SetFrameLevel("1")
	dvlockcheckframe:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT")
	dvlockcheckframe:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT")
	dvlockcheckframe:Hide()
	
-- Lock/Unlock Frames
	dvlockcheck:SetScript("OnClick", function(self,button,down) 
		if self:GetChecked(true) then
			dvlockcheckframe:Hide()
		else
			dvlockcheckframe:Show()
		end
	end)
	
--Open Categories Fix
do
	local function get_panel_name(panel)
		local tp = type(panel)
		local cat = INTERFACEOPTIONS_ADDONCATEGORIES
		if tp == "string" then
			for i = 1, #cat do
				local p = cat[i]
				if p.name == panel then
					if p.parent then
						return get_panel_name(p.parent)
					else
						return panel
					end
				end
			end
		elseif tp == "table" then
			for i = 1, #cat do
				local p = cat[i]
				if p == panel then
					if p.parent then
						return get_panel_name(p.parent)
					else
						return panel.name
					end
				end
			end
		end
	end

	local function InterfaceOptionsFrame_OpenToCategory_Fix(panel)
		if doNotRun or InCombatLockdown() then return end
		local panelName = get_panel_name(panel)
		if not panelName then return end -- if its not part of our list return early
		local noncollapsedHeaders = {}
		local shownpanels = 0
		local mypanel
		local t = {}
		local cat = INTERFACEOPTIONS_ADDONCATEGORIES
		for i = 1, #cat do
			local panel = cat[i]
			if not panel.parent or noncollapsedHeaders[panel.parent] then
				if panel.name == panelName then
					panel.collapsed = true
					t.element = panel
					InterfaceOptionsListButton_ToggleSubCategories(t)
					noncollapsedHeaders[panel.name] = true
					mypanel = shownpanels + 1
				end
				if not panel.collapsed then
					noncollapsedHeaders[panel.name] = true
				end
				shownpanels = shownpanels + 1
			end
		end
		local Smin, Smax = InterfaceOptionsFrameAddOnsListScrollBar:GetMinMaxValues()
		if shownpanels > 15 and Smin < Smax then 
		  local val = (Smax/(shownpanels-15))*(mypanel-2)
		  InterfaceOptionsFrameAddOnsListScrollBar:SetValue(val)
		end
		doNotRun = true
		InterfaceOptionsFrame_OpenToCategory(panel)
		doNotRun = false
	end

	hooksecurefunc("InterfaceOptionsFrame_OpenToCategory", InterfaceOptionsFrame_OpenToCategory_Fix)
end

--DV Slash Setup
local RegisteredEvents = {};
local dvslash = CreateFrame("Frame", addonName.."Slash", UIParent)

dvslash:SetScript("OnEvent", function (self, event, ...) 
	if (RegisteredEvents[event]) then 
	return RegisteredEvents[event](self, event, ...) 
	end
end)

function RegisteredEvents:ADDON_LOADED(event, addon, ...)
	if (addon == addonName) then
		_G["SLASH_"..addonName.."1"] = '/dvc'
		SlashCmdList[addonName] = function (msg, editbox)
			addonTab.SlashCmdHandler(msg, editbox)	
		end
		DEFAULT_CHAT_FRAME:AddMessage(addonName.." loaded successfully. Type /dvc for usage",0,192,255)
	end
end

for k, v in pairs(RegisteredEvents) do
	dvslash:RegisterEvent(k)
end

function addonTab.ShowHelp()
	DEFAULT_CHAT_FRAME:AddMessage(" ")
	DEFAULT_CHAT_FRAME:AddMessage(addonName.." Quick commands list (/dvc):")
	DEFAULT_CHAT_FRAME:AddMessage("  /dvc (c)onfig: Opens the "..addonName.." Interface Options menu.")
	DEFAULT_CHAT_FRAME:AddMessage("  /dvc (u)nlock: Unlocks all "..addonName.." frames.")
	DEFAULT_CHAT_FRAME:AddMessage("  /dvc (l)ock: Locks all "..addonName.." frames.")
	DEFAULT_CHAT_FRAME:AddMessage("  /dvc reset: Resets "..addonName.." to defaults.")
	DEFAULT_CHAT_FRAME:AddMessage('Example: Typing "/dvc u" or "/dvc unlock" will unlock '..addonName..'.')
	DEFAULT_CHAT_FRAME:AddMessage(" ")
end

function addonTab.SetConfigToDefaults()
	DEFAULT_CHAT_FRAME:AddMessage("Resetting config to defaults")
	DejaViewClassicDBPC = DefaultConfig
	RELOADUI()
end

function addonTab.GetConfigValue(key)
	return DejaViewClassicDBPC[key]
end

function addonTab.PrintPerformanceData()
	UpdateAddOnMemoryUsage()
	local mem = GetAddOnMemoryUsage(addonName)
	DEFAULT_CHAT_FRAME:AddMessage(addonName.." is currently using " .. mem .. " kbytes of memory")
	collectgarbage(collect)
	UpdateAddOnMemoryUsage()
	mem = GetAddOnMemoryUsage(addonName)
	DEFAULT_CHAT_FRAME:AddMessage(addonName.." is currently using " .. mem .. " kbytes of memory after garbage collection")
end

function addonTab.SlashCmdHandler(msg, editbox)
	--DEFAULT_CHAT_FRAME:AddMessage("command is " .. msg .. "\n")
	if (string.lower(msg) == "config") or (string.lower(msg) == "c") then
		InterfaceOptionsFrame_OpenToCategory(addonName);
	elseif (string.lower(msg) == "dumpconfig") then
		DEFAULT_CHAT_FRAME:AddMessage("With defaults")
		for k,v in pairs(DVDefaultConfig) do
			DEFAULT_CHAT_FRAME:AddMessage(k,addonTab.GetConfigValue(k))
		end
		DEFAULT_CHAT_FRAME:AddMessage("Direct table")
		for k,v in pairs(DVDefaultConfig) do
			DEFAULT_CHAT_FRAME:AddMessage(k,v)
		end
	elseif (string.lower(msg) == "lock") or (string.lower(msg) == "l") then
		dvlockcheckframe:Hide()
		dvlockcheck:SetChecked(true)
	elseif (string.lower(msg) == "unlock") or (string.lower(msg) == "u") then
		dvlockcheckframe:Show()
		dvlockcheck:SetChecked(false)
	elseif (string.lower(msg) == "reset") then
		DejaViewClassicDBPC = addonTab.defaults;
		ReloadUI();
	elseif (string.lower(msg) == "perf") then
		addonTab.PrintPerformanceData()
	else
		addonTab.ShowHelp()
	end
end
	SlashCmdList[addonName] = addonTab.SlashCmdHandler;